﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MemoryPatch
{

    public class AddressFoundEventArgs : EventArgs
    {
        public int Address { get; private set; }
        public byte[] Value { get; private set; }
        public byte[] ValueA { get; private set; }
        public byte[] ValueB { get; private set; }

        public DataType DataType { get; private set; }

        public AddressFoundEventArgs(int address, byte[] value, DataType data, int saveIndex, byte[] valueA, byte[] valueB)
        {
            if (saveIndex > 2 || saveIndex < 0)
                throw new ArgumentException("save Index can only be between 0 and 2");
            
            Value = (byte[])value.Clone();

            //presurve the values from the last search
            ValueA = valueA;
            ValueB = valueB;

            //save the a and b values for other searches
            if (saveIndex == 1)                            
                ValueA = (byte[])value.Clone();
            else if (saveIndex == 2)
                ValueB = (byte[])value.Clone();            

            Address = address;
            DataType = data;
        }

        public string GetStringValue()
        {
            switch (DataType)
            {
                case DataType.Int32:
                    return BitConverter.ToInt32(Value, 0).ToString();                    
                case DataType.Int16:
                    return BitConverter.ToInt16(Value, 0).ToString();                   
                case DataType.Byte:
                    return ((sbyte)Value[0]).ToString();                    
                case DataType.UInt32:
                    return BitConverter.ToUInt32(Value, 0).ToString();                    
                case DataType.UInt16:
                    return BitConverter.ToUInt16(Value, 0).ToString();                    
                case DataType.UByte:
                    return ((byte)Value[0]).ToString();                    
                case DataType.String:
                    ASCIIEncoding encoding = new ASCIIEncoding();
                    return encoding.GetString(Value);
                case DataType.Float:
                    return BitConverter.ToSingle(Value, 0).ToString();
                case DataType.Double:
                    return BitConverter.ToDouble(Value, 0).ToString();
                default:
                    throw new Exception("Unknown DataType");
            }
        }
    }

    public class UpdateCountEventArgs : EventArgs
    {
        public int Count { get; private set; }

        public UpdateCountEventArgs(int count)
        {
            Count = count;
        }
    }
}
