﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Microsoft.Win32;
using System.IO;
using System.Threading;
using System.Xml.Serialization;
using System.Reflection;

namespace MemoryPatch
{   
    public partial class Form1 : Form
    {
        #region Fields
        private SelectProcess frmSelect = new SelectProcess();
        private Process _process;
        private ProcessModule _module;
        private SearchMemory _seracher;        
        private SearchData _currentSearchData;
        private Dictionary<int, SavedAddress> _savedAddresses = 
            new Dictionary<int, SavedAddress>();
        private int _lastAddress = -1;

        private Dictionary<string, List<DataGridViewRow>> _rows = 
            new Dictionary<string, List<DataGridViewRow>>();
        private bool _updateing = false;
        private ContextMenu _dataTypeMenu = new ContextMenu();
        private string _lastLoaded;

        private bool _forceRefesh = false;
        //locking stuff
        private Dictionary<int, LockData> _lockList = new Dictionary<int, LockData>();
        private static object _lock = new object();
        private Thread _lockUpdate;
        private ConfigData _config;
        private Exception _lastException = null;
        private bool _saveingAResults = false;
        private bool _saveingBResults = false;
        #endregion       

        public void SetNewProcess(Process process, ProcessModule module)
        {           
            if (process != null && module != null)
            {
                _process = process;
                _module = module;

                lbActiveProcess.Text = _process.ProcessName + " (" + _module.ModuleName + ")";
                timer1.Enabled = true;

                if (_seracher != null)
                {
                    _seracher.OnValueFound -= new EventHandler<AddressFoundEventArgs>(Seracher_OnValueFound);
                    _seracher.OnProgressChange -= new EventHandler<SearchUpdateEventArgs>(Seracher_OnProgressChange);
                }
                _seracher = new SearchMemory(_process, _module, this);
                _seracher.OnValueFound += new EventHandler<AddressFoundEventArgs>(Seracher_OnValueFound);
                _seracher.OnProgressChange += new EventHandler<SearchUpdateEventArgs>(Seracher_OnProgressChange);
            }
            else
            {
                lbActiveProcess.Text = "None";
                timer1.Enabled = false;
                _process = null;
                _module = null;
            }
        }

        #region Load
        public Form1()
        {
            InitializeComponent();  
            //ProcessStartInfo info = new ProcessStartInfo(@"C:\Documents and Settings\Shawn200\My Documents\My Downloads\snes\zsnesw.exe");
            //_process = Process.Start(info);
            

            _lockUpdate = new Thread(new ThreadStart(delegate()
                {
                    while (true)
                    {
                        lock (_lock)
                        {
                            foreach (LockData locked in _lockList.Values)
                            {
                                if(_seracher != null)
                                    _seracher.WriteValue(locked.Address, locked.Data);
                            }
                        }
                        Thread.Sleep(50);
                    }
                }));
            _lockUpdate.IsBackground = true;
            _lockUpdate.Name = "Lock Memory";
            _lockUpdate.Start();                       
        }

        private void LoadConfig()
        {
            Stream stream = null;
            try
            {
                XmlSerializer objXmlSer = new XmlSerializer(typeof(ConfigData));
                string path = Path.GetDirectoryName(Application.ExecutablePath);
                stream = File.Open(path + "\\config.xml", FileMode.Open);
                _config = (ConfigData)objXmlSer.Deserialize(stream);
                stream.Close();

                Left = _config.X;
                Top = _config.Y;
                Width = _config.Width;
                Height = _config.Height;
            }
            catch (Exception ex)
            {
                _config = new ConfigData();
            }
            finally
            {
                if (stream != null)
                    stream.Close();
            }
        }

        private void SaveConfig()
        {
            Stream stream = null;
            try
            {
                XmlSerializer objXmlSer = new XmlSerializer(typeof(ConfigData));
                string path = Path.GetDirectoryName(Application.ExecutablePath);
                stream = File.Open(path+ "\\config.xml", FileMode.OpenOrCreate);
                if (WindowState == FormWindowState.Normal)
                {
                    _config.X = Left;
                    _config.Y = Top;
                    _config.Width = Width;
                    _config.Height = Height;
                }
                objXmlSer.Serialize(stream, _config);
                stream.Close();
            }
            catch (Exception ex)
            {
                _config = new ConfigData();
            }
            finally
            {
                if (stream != null)
                    stream.Close();
            }
        }     

        private void Form1_Load(object sender, EventArgs e)
        {
            cboSearch.Items.Add(SearchType.Excat.ToString());            
            cboSearch.Items.Add(SearchType.UnKnown.ToString());

            cboNextSearch.Items.Add(SearchType.Excat.ToString());
            cboNextSearch.Items.Add(SearchType.HasChanged.ToString());
            cboNextSearch.Items.Add(SearchType.HasNotChanged.ToString());
            cboNextSearch.Items.Add(SearchType.HasIncreased.ToString());
            cboNextSearch.Items.Add(SearchType.HasDecreased.ToString());
            cboNextSearch.Items.Add(SearchType.HasDecreasedBy.ToString());
            cboNextSearch.Items.Add(SearchType.HasIncreasedBy.ToString());

            string[] datatypes = Enum.GetNames(typeof(DataType));
            foreach (string type in datatypes)
            {
                cboDataType.Items.Add(type);
                _dataTypeMenu.MenuItems.Add(type);
            }

            cboDataType.SelectedIndex = 0;
            cboSearch.SelectedIndex = 0;
            cboNextSearch.SelectedIndex = 0;

            LoadConfig();
        }
        #endregion

        #region Search Events
        private void Seracher_OnProgressChange(object sender, SearchUpdateEventArgs e)
        {
            pgSearching.Value = e.PrecentDone;
            lbFoundCount.Text = e.AddressFoundCount.ToString();

            if (e.PrecentDone == 100)
            {
                if (_saveingAResults)
                {
                    btnSearchA.Enabled = true;
                    txtADisc.Enabled = true;
                    _saveingAResults = false;
                }

                if (_saveingBResults)
                {
                    btnSearchB.Enabled = true;
                    txtBDisc.Enabled = true;
                    _saveingBResults = false;
                }
                groupNextSearch.Enabled = true;
                lbDataType.Text = "Data Type " + _currentSearchData.DataType.ToString();
                pgSearching.Value = 0;
            }

        }

        public void Seracher_OnValueFound(object sender, AddressFoundEventArgs e)
        {
            DataGridViewRow row = new DataGridViewRow();

            row.CreateCells(dataResults, string.Format("{0:X000000}", e.Address),
                                        e.GetStringValue(),
                                        e.DataType.ToString());
            dataResults.Rows.Add(row);
        }
      
        #endregion

        #region Selecting Process

        private void FrmSelect_OnProcessSelected(object sender, ProcessEventArgs e)
        {
            lbActiveProcess.Text = e.Process.ProcessName;
            _process = e.Process;
            SetNewProcess(e.Process, e.Module);
            timer1.Enabled = true;
        }

        private void btnSelectProcess_Click(object sender, EventArgs e)
        {
            //if (frmSelect != null)
            //{
            //    frmSelect.OnProcessSelected -= new EventHandler<ProcessEventArgs>(FrmSelect_OnProcessSelected);
            //    frmSelect.Dispose();
            //}
            if(frmSelect == null || frmSelect.IsDisposed)
                frmSelect = new SelectProcess();

            //frmSelect.OnProcessSelected += new EventHandler<ProcessEventArgs>(FrmSelect_OnProcessSelected);
            if (frmSelect.ShowDialog() == DialogResult.OK)
            {
                lbActiveProcess.Text = frmSelect.Args.Process.ProcessName;
                _process = frmSelect.Args.Process;
                SetNewProcess(frmSelect.Args.Process, frmSelect.Args.Module);
                timer1.Enabled = true;
            }
        }

        #endregion       

        #region Searching
        private void btnFirstSearch_Click(object sender, EventArgs e)
        {
            if (_seracher != null)
            {

                SearchType searchType = (SearchType)Enum.Parse(typeof(SearchType), cboSearch.Text);
                if (searchType == SearchType.Excat && txtValue.Text == "")
                    MessageBox.Show("Invalid input");
                else
                {
                    DataType datatype = (DataType)Enum.Parse(typeof(DataType), cboDataType.Text);                    

                    _currentSearchData = SearchData.CreateSearchData(true, searchType, datatype, txtValue.Text, 0);
                    ResetSearchData();

                    _seracher.FirstSearch(_currentSearchData);
                }
            }
        }

        private void btnNextSearch_Click(object sender, EventArgs e)
        {
            if (_seracher != null)
            {

                SearchType searchType = (SearchType)Enum.Parse(typeof(SearchType), cboNextSearch.Text);
                if (searchType == SearchType.Excat && txtNextVal1.Text == "")
                    MessageBox.Show("Invalid input");
                else
                {
                    DataType datatype = _currentSearchData.DataType;
                    
                    int saveIndex = 0;
                    if (chkSaveAsA.Checked)
                    {
                        _saveingAResults = true;
                        _saveingBResults = false;
                        saveIndex = 1;
                    }
                    else if (chkSaveAsB.Checked)
                    {
                        _saveingAResults = false;
                        _saveingBResults = true;
                        saveIndex = 2;
                    }
                    _currentSearchData = SearchData.CreateSearchData(true, searchType, datatype, txtNextVal1.Text, saveIndex);
                    ResetSearchData();

                    _seracher.NextSearch(_currentSearchData);
                }
            }
        }

        private void btnSearchA_Click(object sender, EventArgs e)
        {
            if (_seracher != null)
            {
                SearchType searchType = SearchType.SearchForAValue;
                DataType datatype = _currentSearchData.DataType;

                _currentSearchData = SearchData.CreateSearchData(true, searchType, datatype, txtNextVal1.Text, 0);
                ResetSearchData();

                _seracher.NextSearch(_currentSearchData);
            }
        }

        private void btnSearchB_Click(object sender, EventArgs e)
        {
            if (_seracher != null)
            {
                SearchType searchType = SearchType.SearchForBValue;
                DataType datatype = _currentSearchData.DataType;

                _currentSearchData = SearchData.CreateSearchData(true, searchType, datatype, txtNextVal1.Text, 0);
                ResetSearchData();

                _seracher.NextSearch(_currentSearchData);
            }
        }

        private void ResetSearchData()
        {
            dataResults.Rows.Clear();            
            lbFoundCount.Text = "0";
        }

        private void btnCancle_Click(object sender, EventArgs e)
        {
            _seracher.CancleSearch();         
        }

        private void txtValue_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                btnFirstSearch_Click(sender, e);
            }
        }

        private void txtNextVal1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                btnNextSearch_Click(sender, e);
            }
        }                       
        #endregion
               
        #region Grid View Functinos
        private int GetAddressOfCell(int rowIndex)
        {
            if (dataAccepted.Rows.Count <= rowIndex)
                return -1;

            DataGridViewRow selRow = dataAccepted.Rows[rowIndex];
            object val = selRow.Cells["colAccAddress"].Value;
            if (val != null)
            {
                string addressStr = val.ToString().Trim();
                int address = Int32.Parse(addressStr, System.Globalization.NumberStyles.AllowHexSpecifier, null);
                return address;
            }
            else
            {
                return -1;
            }
        }

        private void dataAccepted_CellBeginEdit(object sender, DataGridViewCellCancelEventArgs e)
        {
            _lastAddress = GetAddressOfCell(e.RowIndex);
        }      

        private void dataAccepted_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                if (dataAccepted.SelectedCells.Count > 0 && !_updateing)
                {
                    DataGridViewCell[] data = new DataGridViewCell[dataAccepted.SelectedCells.Count];
                    dataAccepted.SelectedCells.CopyTo(data, 0);
                    foreach (DataGridViewCell cell in data)
                    {
                        if (cell.ColumnIndex != 0)
                            continue;

                        int address = GetAddressOfCell(cell.RowIndex);//_selectedAddress.Address;
                        if (_savedAddresses.ContainsKey(address))
                        {
                            _savedAddresses.Remove(address);
                        }                       
                        lock (_lock)
                        {
                            //if it's locked and it's not in the locked list add it
                            if (_lockList.ContainsKey(address))
                            {
                                _lockList.Remove(address);
                            }
                        }
                    }
                }
            }
        }      

        private void dataAccepted_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex != -1 && e.RowIndex != -1 && !_updateing)
            {
                int address = GetAddressOfCell(e.RowIndex);

                //is the user is filling in data but has not filled in the address yet
                if (address == -1)
                    return;

                SavedAddress data;
                _savedAddresses.TryGetValue(address, out data);

                if (dataAccepted.Columns[e.ColumnIndex].Name == "colAccAddress")
                {

                    if (!_savedAddresses.ContainsKey(address))
                    {
                        _savedAddresses.Add(address, new SavedAddress(false, "New", address, "0", DataType.UByte, cboFilter.Text));
                        data = _savedAddresses[address];

                        //remove the last address
                        if (_lastAddress != -1)
                        {                             
                            _savedAddresses.Remove(_lastAddress);                            
                        }
                    }
                    else
                    {
                        MessageBox.Show("Address already Exists.");
                        dataAccepted.Rows.RemoveAt(e.RowIndex);                                                
                        return;
                    }
                }
                else if (dataAccepted.Columns[e.ColumnIndex].Name == "colAccValue")
                {                    
                    if (data != null)
                    {                       
                        string newVal = null;
                        DataGridViewRow selRow = dataAccepted.Rows[e.RowIndex];
                        object val = selRow.Cells["colAccValue"].Value;
                        if (val != null)
                            data.StringValue = val.ToString();

                        int written = _seracher.WriteValue(address, data.Value);

                        lock (_lock)
                        {
                            //if it's locked and it's in the locked list update the value
                            if (_lockList.ContainsKey(address))
                            {
                                _lockList.Remove(address);
                                _lockList.Add(address, new LockData(address, data.Value));
                            }
                        }
                    }
                }
                else if(dataAccepted.Columns[e.ColumnIndex].Name == "colAccDataType")
                {
                    DataGridViewRow selRow = dataAccepted.Rows[e.RowIndex];
                    string dataType = (string)selRow.Cells["colAccDataType"].Value;                    
                    try
                    {
                        DataType type = (DataType)Enum.Parse(typeof(DataType), dataType);
                        _forceRefesh = true;
                    }
                    catch (Exception ex)
                    {
                        string[] types = Enum.GetNames(typeof(DataType));
                        string typeList = "";
                        foreach (string  type in types)
	                    {
                            typeList += type + ",";
                        }
                        selRow.Cells["colAccDataType"].Value = "UByte";
                        MessageBox.Show("Unkown Type try: " + typeList);
                    }                    
                }
               
                //RefeshRow(data, dataAccepted.Rows[e.RowIndex]);
                if(data != null)
                    RefeshAddress(dataAccepted.Rows[e.RowIndex], data);                
            }            
        }         

        private void dataResults_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if(e.RowIndex == -1)
                return;

            DataGridViewRow selRow = dataResults.Rows[e.RowIndex];
            DataGridViewRow newRow = new DataGridViewRow();                         
            object val = selRow.Cells["colAddress"].Value;
            int address = Int32.Parse(val.ToString(), System.Globalization.NumberStyles.AllowHexSpecifier, null);                            

            DataType dataType = (DataType)Enum.Parse(typeof(DataType), selRow.Cells["colType"].Value.ToString());

            if (!_savedAddresses.ContainsKey(address))
            {
                string value;
                if (dataType == DataType.String)
                {
                    ASCIIEncoding encoding = new ASCIIEncoding();
                    object strVal = selRow.Cells["colValue"].Value;
                    int count = encoding.GetByteCount(strVal.ToString());

                    value = _seracher.ReadMemoryAsString(address, dataType, count);

                }
                else
                {
                    value = _seracher.ReadMemoryAsString(address, dataType, null);
                }

                SavedAddress savedAddress;
                if (cboFilter.Text == "*")
                {
                    savedAddress = new SavedAddress(false, "New", address, value, dataType, "Default");
                }
                else
                {
                    savedAddress = new SavedAddress(false, "New", address, value, dataType, cboFilter.Text);

                    
                }
                _savedAddresses.Add(address, savedAddress);
                newRow.CreateCells(dataAccepted, savedAddress.Locked,
                    savedAddress.Name,
                    savedAddress.StringAddress,
                    savedAddress.StringValue,
                    savedAddress.DataType,
                    savedAddress.Category);

                RefeshRow(savedAddress, newRow);
                //_savedAddresses.Add(address, new SavedAddress(dataType, value, address, "New", false, dataAccepted2.Rows.Count));
                dataAccepted.Rows.Add(newRow);
            }
            else
            {
                MessageBox.Show("The address has already been added. (" + selRow.Cells["colAddress"].Value + ")");
            }

        }
        #endregion        

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (_seracher == null)
                return;
            //try
            //{
                for (int i = 0; i < dataAccepted.Rows.Count; i++)
                {
                    int address = GetAddressOfCell(i);
                    if (address != -1)
                    {
                        SavedAddress data = _savedAddresses[address];
                        _updateing = true;
                       
                        bool locked = data.Locked; //GetLockOfCell(i);
                        if (locked)
                        {
                            lock (_lock)
                            {
                                //if it's locked and it's not in the locked list add it
                                if (!_lockList.ContainsKey(address))
                                    _lockList.Add(address, new LockData(address, data.Value));
                                    //_lockList.Add(address, new LockData(address, GetByteValueOfCell(i)));                                
                            }
                        }
                        else
                        {
                            lock (_lock)
                            {
                                //if it's not locked and it's in the locked list remove it
                                if (_lockList.ContainsKey(address))
                                    _lockList.Remove(address);
                            }
                            string currentValue = data.StringValue;//GetValueOfCell(i);
                            string newValue;
                            DataType dataType = data.DataType;//GetDataTypeOfCell(i);
                            if (dataType == DataType.String)
                            {
                                ASCIIEncoding encoding = new ASCIIEncoding();
                                int count = encoding.GetByteCount(currentValue);
                                lock (_lock)
                                {
                                    newValue = _seracher.ReadMemoryAsString(address, dataType, count);
                                }
                            }
                            else
                            {
                                newValue = _seracher.ReadMemoryAsString(address, dataType, null);
                            }

                            if (newValue != currentValue || _forceRefesh)
                            {
                                data.StringValue = newValue;
                                RefeshRow(data, dataAccepted.Rows[i]);                                                                
                            }
                        }
                        _updateing = false;
                    }
                }
                _forceRefesh = false;    
            //}
            //catch (Exception ex)
            //{
            //    if (_lastException == null ||
            //        _lastException.Message != ex.Message)
            //    {                    
            //        MessageBox.Show(ex.Message);
            //        _lastException = ex;
            //    }
            //    else
            //    {
            //        _lastException = ex;
            //    }
            //}
        }

        #region Data Accepted Buttons
        private void btnClearAll_Click(object sender, EventArgs e)
        {
            lock (_lock)
            {                
                _lockList.Clear();
            }
            dataAccepted.Rows.Clear();
            _savedAddresses.Clear();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog save = new SaveFileDialog();
            if (_lastLoaded != null)
                save.InitialDirectory = _lastLoaded;
            else
                save.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\saves";
            save.Filter = "*.txt|*.txt";
            if (save.ShowDialog() == DialogResult.OK)
            {
                Save(save.FileName);
                _lastLoaded = Path.GetDirectoryName(save.FileName);
            }
        }

        private void Save(string file)
        {
            StreamWriter w = null;
            try
            {
                if (File.Exists(file))
                    File.Delete(file);

                w = new StreamWriter(file);
                foreach (SavedAddress data in _savedAddresses.Values)
                {
                    string line = data.Locked + ";" +
                                 data.Name + ";" +
                                 data.StringAddress + ";" +
                                 data.StringValue + ";" +
                                 data.DataType + ";" +
                                 data.Category;
                     w.WriteLine(line);
                }
                //int count = dataAccepted2.Rows.Count - 1;               
                //for (int i = 0; i < count; i++)
                //{
                //    int x = GetAddressOfCell(i);
                //    string line = GetLockOfCell(i) + ";" +
                //                  GetNameOfCell(i) + ";" +
                //                  string.Format("{0:X000000}", x) + ";" +
                //                  GetValueOfCell(i) + ";" +
                //                  GetDataTypeOfCell(i);
                //    w.WriteLine(line);                
                //}
            }
            catch (Exception Exception)
            {
                MessageBox.Show(Exception.Message);
            }
            finally
            {
                if (w != null)
                    w.Close();
            }
        }
        
        private void btnOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog open = new OpenFileDialog();
            if (_lastLoaded != null)
                open.InitialDirectory = _lastLoaded;
            else
                open.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + "\\saves";
            open.Filter = "*.txt|*.txt";
            if (open.ShowDialog() == DialogResult.OK)
            {
                Open(open.FileName);
                _lastLoaded = Path.GetDirectoryName(open.FileName);
            }
        }

        private void Open(string file)
        {
            StreamReader r = null;
            string line;
            //try
            //{
                r = new StreamReader(file);

                //int count = int.Parse(r.ReadLine());
                btnClearAll_Click(null, null);
                //for (int i = 0; i < count; i++)
                line = r.ReadLine();                
                //List<DataGridViewRow> list;
                string category = "Default";
                cboFilter.Items.Clear();
                cboFilter.Items.Add("*");

                while(line != null)
                {
                    string[] lineSplit = line.Split(new char[] { ';' });


                    //if (lineSplit.Length != 6)
                    //{
                    //    category = line;
                    //    cboFilter.Items.Add(line);
                    //    line = r.ReadLine();
                    //    continue;
                    //}
                    //else
                    //{
                        line = r.ReadLine();
                    //}
                    SavedAddress savedAddress = new SavedAddress(
                                          bool.Parse(lineSplit[0]),
                                          lineSplit[1],
                                          lineSplit[2],
                                          lineSplit[3],
                                          (DataType)Enum.Parse(typeof(DataType), lineSplit[4]),
                                          lineSplit[5]);

                    if (!cboFilter.Items.Contains(lineSplit[5]))
                    {
                        cboFilter.Items.Add(lineSplit[5]);
                    }

                    if (!_savedAddresses.ContainsKey(savedAddress.Address))
                    {
                        _savedAddresses.Add(savedAddress.Address, savedAddress);
                    }

                        //DataGridViewRow newRow = new DataGridViewRow();
                        //newRow.CreateCells(dataAccepted2,
                        //   bool.Parse(lineSplit[0]),
                        //              lineSplit[1],
                        //              lineSplit[2],
                        //              lineSplit[3],
                        //              lineSplit[4]);                        
                        //dataAccepted2.Rows.Add(newRow);                      
                }
                RefeshDataGrid();
            //}
            //catch (Exception Exception)
            //{
            //    MessageBox.Show(Exception.Message);
            //}
            //finally
            //{
                if (r != null)
                    r.Close();
            //}
        }

        private void RefeshRow(SavedAddress address, DataGridViewRow row)
        {
            row.Cells[0].Value = address.Locked;
            row.Cells[1].Value = address.Name;
            row.Cells[2].Value = address.StringAddress;
            row.Cells[3].Value = address.StringValue;
            row.Cells[4].Value = address.DataType;
        }

        private void RefeshAddress(DataGridViewRow row, SavedAddress address)
        {
            _updateing = true;
            if (row.Cells[0].Value != null)
                address.Locked = (bool)row.Cells[0].Value;
            else
                row.Cells[0].Value = address.Locked;

            if (row.Cells[1].Value != null)
                address.Name = (string)row.Cells[1].Value;
            else
                row.Cells[1].Value = address.Name;

            if (row.Cells[2].Value != null)
                address.StringAddress = (string)row.Cells[2].Value;
            else
                row.Cells[2].Value = address.StringAddress;

            if (row.Cells[3].Value != null)
                address.StringValue = (string)row.Cells[3].Value;
            else
                row.Cells[3].Value = address.StringValue;

            if (row.Cells[4].Value != null)
                address.DataType = (DataType)Enum.Parse(typeof(DataType), row.Cells[4].Value.ToString());
            else
                row.Cells[4].Value = address.DataType;
            _updateing = false;
        }

        private void RefeshDataGrid()
        {
            dataAccepted.Rows.Clear();
            foreach (SavedAddress address in _savedAddresses.Values)
            {
                if (address.Category == cboFilter.Text || cboFilter.Text =="*")
                {
                    DataGridViewRow newRow = new DataGridViewRow();
                    newRow.CreateCells(dataAccepted,
                                  address.Locked,
                                  address.Name,
                                  address.StringAddress,
                                  address.StringValue,
                                  address.DataType);
                    dataAccepted.Rows.Add(newRow);
                }
            }
        }
        #endregion

        private void btnRun_Click(object sender, EventArgs e)
        {
            OpenFileDialog open = new OpenFileDialog();
            open.InitialDirectory = _config.LastRunDir;
            open.Filter = "*.exe|*.exe";
            if (open.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _config.LastRunDir = Path.GetDirectoryName(open.FileName);
                    ProcessStartInfo info = new ProcessStartInfo(open.FileName);
                    _process = Process.Start(info);
                    SetNewProcess(_process, _process.MainModule);
                    timer1.Enabled = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error opening try again.");
                    _process.Kill();
                }
            }
        }

        private void cboFilter_SelectedIndexChanged(object sender, EventArgs e)
        {            
            RefeshDataGrid();
        }      

        private void btnSaveSearch_Click(object sender, EventArgs e)
        {

        }

        private void btnOpenSearch_Click(object sender, EventArgs e)
        {

        }

        private void chkSaveAsA_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSaveAsB.Checked && chkSaveAsA.Checked)
                chkSaveAsB.Checked = false;
        }

        private void chkSaveAsB_CheckedChanged(object sender, EventArgs e)
        {
            if (chkSaveAsA.Checked && chkSaveAsB.Checked)
                chkSaveAsA.Checked = false;
        }       
    }

    [XmlRoot("Config")]
    [Serializable]
    public class ConfigData
    {

        [XmlAttribute("x")]
        public int X;
        [XmlAttribute("y")]
        public int Y;
        [XmlAttribute("width")]
        public int Width;
        [XmlAttribute("height")]
        public int Height;
        [XmlAttribute("lastRunDir")]
        public string LastRunDir;
    }
}
