﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace MemoryPatch
{
    public class SearchMemory
    {
        [DllImport("kernel32.dll", EntryPoint = "ReadProcessMemory")]
        public static extern Int32 ReadProcessMemory(IntPtr hProcess, int lpBaseAddress,
            byte[] buffer, int size, out int lpNumberOfBytesRead);

        [DllImport("kernel32.dll", EntryPoint = "WriteProcessMemory")]
        public static extern Int32 WriteProcessMemory(IntPtr hProcess, int lpBaseAddress,
            byte[] buffer, int size, out int lpNumberOfBytesRead);

        [DllImport("kernel32.dll", EntryPoint = "OpenProcess")]
        public static extern IntPtr OpenProcess(int dwDesiredAccess, bool bInheritHandle, int dbProcessId);

        public const int PROCESS_VM_READ = 0x0010;
        public const int PROCESS_VM_WRITE = 0x0020;
        private Process _process;
        private ProcessModule _module;
        private Thread _searchThread;
        private Form _parent;
        private List<AddressFoundEventArgs> _found1 = new List<AddressFoundEventArgs>();
        private List<AddressFoundEventArgs> _found2 = new List<AddressFoundEventArgs>();
        private bool _useFound1 = true;
        public event EventHandler OnCancel;
        public event EventHandler<AddressFoundEventArgs> OnValueFound;        
        public event EventHandler<SearchUpdateEventArgs> OnProgressChange;
        private static object _processLock = new object();

        #region Properties
        public Process Process
        {
            get
            {
                lock(_processLock)
                    return _process;
            }
            set
            {
                lock (_processLock)
                {
                    _process = value;
                }
            }

        }

        public ProcessModule Module
        {
            get
            {
                lock (_processLock)
                    return _module;
            }
            set
            {
                lock (_processLock)
                {
                    _module = value;
                }
            }

        }
        #endregion
        public SearchMemory(Process process, ProcessModule module, Form parent)
        {
            if (process == null)
                throw new ArgumentNullException("process");

            if (module == null)
                throw new ArgumentNullException("module");

            Process = process;
            Module = module;
            _parent = parent;
        }

        public void CancleSearch()
        {
            while (_searchThread != null && _searchThread.ThreadState == 0)
            {
                _searchThread.Abort();
                Thread.Sleep(100);
            }

            //final update
            UpdateProgress(0, 1, 1, 1, 0);
        }

        public void FirstSearch(SearchData data)
        {            
            CancleSearch();
            
            _searchThread = new Thread(new ParameterizedThreadStart(FirstSearch));  
            _searchThread.Name = "SerachingThread";            

            _searchThread.Start(data);
        }

        public void NextSearch(SearchData data)
        {
            CancleSearch();

            _searchThread = new Thread(new ParameterizedThreadStart(NextSearch));
            _searchThread.Name = "SerachingThread";

            _searchThread.Start(data);
        }

        private void FirstSearch(object obj)
        {
            try
            {
                SearchData data = (SearchData)obj;

                IntPtr baseAddress = Module.BaseAddress;
                int start = baseAddress.ToInt32();
                int len = Module.ModuleMemorySize;
                int end = start + len;
                IntPtr handle = Process.Handle;
                float lastPrecentDone = 0;

                byte[] value1 = data.Value1;
                byte[] buffer = new byte[data.DataLength];

                //reset how the next search will work
                _found1.Clear();
                _useFound1 = true;

                for (int Address = start; Address < end; Address++)
                {
                    int lengthRead;
                    ReadProcessMemory(handle, Address, buffer, buffer.Length, out lengthRead);
                    if (data.DataType == DataType.Float)
                    {
                        float v1 = BitConverter.ToSingle(buffer, 0);
                        if (float.IsNaN(v1) || float.IsNegativeInfinity(v1))
                            continue;
                    }

                    if (data.DataType == DataType.Double)
                    {
                        double v1 = BitConverter.ToDouble(buffer, 0);
                        if (double.IsNaN(v1) || double.IsNegativeInfinity(v1))
                            continue;
                    }

                    //check it the values match
                    bool match = true;
                    double difference;
                    switch (data.SearchType)
                    {
                        case SearchType.Excat:
                            match = (Compare(buffer, value1, data.DataType, out difference) == CompareType.EqualTo);
                            break;
                        case SearchType.UnKnown:
                            match = true;
                            break;
                        default:
                            throw new Exception("Unknown DataType " + data.SearchType);
                    }

                    //test if this is want we are looking for
                    if (match)
                    {
                        AddressFoundEventArgs args = new AddressFoundEventArgs(Address, buffer,
                                                            data.DataType, data.SaveResultsIndex, null, null);

                        _found1.Add(args);
                        int count = _found1.Count;

                        if (count < 200)
                        {
                            _parent.Invoke(new ThreadStart(delegate()
                                {
                                    if (OnValueFound != null)
                                    {
                                        //OnValueFound(this, new AddressFoundEventArgs(Address, buffer, data.DataType));
                                        OnValueFound(this, args);
                                    }
                                }));
                        }
                    }

                    lastPrecentDone = UpdateProgress(start, end, lastPrecentDone, Address, _found1.Count);
                }

                //final update
                UpdateProgress(start, end, lastPrecentDone, end, _found1.Count);
            }
            catch (ThreadAbortException)
            {
                //just exit
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
        }

        private void NextSearch(object obj)
        {
            try
            {
                SearchData data = (SearchData)obj;

                IntPtr baseAddress = Process.MainModule.BaseAddress;
                int start = baseAddress.ToInt32();
                int len = Process.MainModule.ModuleMemorySize;
                int end = start + len;
                IntPtr handle = Process.Handle;
                float lastPrecentDone = 0;

                byte[] value1 = data.Value1;                
                byte[] buffer = new byte[data.DataLength];
                
                //this handles consective searches
                List<AddressFoundEventArgs> searchList;
                List<AddressFoundEventArgs> saveList;
                if (_useFound1)
                {
                    searchList = _found1;
                    saveList = _found2;
                }
                else
                {
                    searchList = _found2;
                    saveList = _found1;
                }

                saveList.Clear();
                _useFound1 = !_useFound1;

                //foreach (AddressFoundEventArgs foundAddress in searchList)
                for (int i = 0; i < searchList.Count; i++)
                {
                    AddressFoundEventArgs foundAddress = searchList[i];
                    int lengthRead;

                    ReadProcessMemory(handle, foundAddress.Address, buffer, buffer.Length, out lengthRead);

                    if (data.DataType == DataType.Float)
                    {
                        float v1 = BitConverter.ToSingle(buffer, 0);
                        if (float.IsNaN(v1) || float.IsNegativeInfinity(v1))
                        {
                            searchList.RemoveAt(i);
                            i--;
                            continue;
                        }
                    }

                    if (data.DataType == DataType.Double)
                    {
                        double v1 = BitConverter.ToDouble(buffer, 0);
                        if (double.IsNaN(v1) || double.IsNegativeInfinity(v1))
                        {
                            searchList.RemoveAt(i);
                            i--;
                            continue;
                        }
                    }

                    try
                    {
                        double diff;
                        bool match = true;                      
                        switch (data.SearchType)
                        {
                            case SearchType.Excat:
                                match = (Compare(buffer, value1, data.DataType, out diff) == CompareType.EqualTo);
                                break;
                            case SearchType.HasIncreased:
                                match = (Compare(buffer, foundAddress.Value, data.DataType, out diff) == CompareType.GreaterThen);
                                break;
                            case SearchType.HasDecreased:
                                match = (Compare(buffer, foundAddress.Value, data.DataType, out diff) == CompareType.LessThen);
                                break;
                            case SearchType.HasNotChanged:
                                match = (Compare(buffer, foundAddress.Value, data.DataType, out diff) == CompareType.EqualTo);
                                break;
                            case SearchType.HasChanged:
                                match = (Compare(buffer, foundAddress.Value, data.DataType, out diff) != CompareType.EqualTo);
                                break;
                            case SearchType.HasIncreasedBy:
                                match = (Compare(buffer, foundAddress.Value, data.DataType, out diff) != CompareType.GreaterThen);
                                match = match && (diff == data.Difference);
                                break;
                            case SearchType.HasDecreasedBy:
                                match = (Compare(buffer, foundAddress.Value, data.DataType, out diff) != CompareType.LessThen);
                                match = match && (diff == data.Difference);
                                break;
                            case SearchType.SearchForAValue:
                                match = (Compare(buffer, foundAddress.ValueA, data.DataType, out diff) == CompareType.EqualTo);                                
                                break;
                            case SearchType.SearchForBValue:
                                match = (Compare(buffer, foundAddress.ValueB, data.DataType, out diff) == CompareType.EqualTo);                                
                                break;
                            default:
                                throw new Exception("Unknown DataType " + data.SearchType);
                        }

                        //test if this is want we are looking for
                        if (match)
                        {
                            //saveList.Add(new AddressFoundEventArgs(foundAddress.Address, buffer, data.DataType, data.SaveResultsIndex));
                            AddressFoundEventArgs args = new AddressFoundEventArgs(foundAddress.Address, buffer,
                                                                            data.DataType, data.SaveResultsIndex, 
                                                                            foundAddress.ValueA, foundAddress.ValueB);

                            saveList.Add(args);
                            int count = saveList.Count;

                            if (count < 400)
                            {
                                _parent.Invoke(new ThreadStart(delegate()
                                {
                                    if (OnValueFound != null)
                                    {
                                        //OnValueFound(this, new AddressFoundEventArgs(foundAddress.Address, buffer, data.DataType));
                                        OnValueFound(this, args);
                                    }
                                }));
                            }

                        }
                    }
                    catch (Exception ex)
                    {
                        throw;
                    }

                    lastPrecentDone = UpdateProgress(start, end, lastPrecentDone, foundAddress.Address, saveList.Count);
                }
                            
                //final update
                UpdateProgress(start, end, lastPrecentDone, end, saveList.Count);
            }
            catch (ThreadAbortException)
            {
                //just exit
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
            }
        }

        public CompareType Compare(byte[] value1, byte[] value2, DataType dataType, out double amoutOfChange)
        {
            CompareType compare;
            switch (dataType)
            {
                case DataType.Int32:
                    {
                        Int32 v1 = BitConverter.ToInt32(value1, 0);
                        Int32 v2 = BitConverter.ToInt32(value2, 0);
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2)
                            return CompareType.LessThen;
                        else if (v1 > v2)
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    }                    
                case DataType.Int16:
                    {

                        Int16 v1 = BitConverter.ToInt16(value1, 0);
                        Int16 v2 = BitConverter.ToInt16(value2, 0);
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2)
                            return CompareType.LessThen;
                        else if (v1 > v2)
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    }     
                case DataType.Byte:
                    {
                        sbyte v1 = (sbyte)value1[0];
                        sbyte v2 = (sbyte)value2[0];
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2)
                            return CompareType.LessThen;
                        else if (v1 > v2)
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    }     
                case DataType.UInt32:
                    {
                        UInt32 v1 = BitConverter.ToUInt32(value1, 0);
                        UInt32 v2 = BitConverter.ToUInt32(value2, 0);
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2)
                            return CompareType.LessThen;
                        else if (v1 > v2)
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    }     
                case DataType.UInt16:
                    {
                        UInt16 v1 = BitConverter.ToUInt16(value1, 0);
                        UInt16 v2 = BitConverter.ToUInt16(value2, 0);
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2)
                            return CompareType.LessThen;
                        else if (v1 > v2)
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    }     
                case DataType.UByte:
                    {
                        byte v1 = value1[0];
                        byte v2 = value2[0];
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2)
                            return CompareType.LessThen;
                        else if (v1 > v2)
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    }     
                case DataType.String:
                    {
                        ASCIIEncoding encoding = new ASCIIEncoding();
                        string v1 = encoding.GetString(value1);
                        string v2 = encoding.GetString(value2);
                        amoutOfChange = 0;
                        if (v1.Equals(v2))
                            return CompareType.EqualTo;
                        else
                            return CompareType.LessThen;
                    }     
                case DataType.Float:
                    {
                        float v1 = BitConverter.ToSingle(value1, 0);
                        float v2 = BitConverter.ToSingle(value2, 0);
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2 || float.IsNaN(v1) || float.IsNegativeInfinity(v1))
                            return CompareType.LessThen;
                        else if (v1 > v2 || float.IsPositiveInfinity(v1))
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    } 
                case DataType.Double:
                    {
                        double v1 = BitConverter.ToDouble(value1, 0);
                        double v2 = BitConverter.ToDouble(value2, 0);
                        amoutOfChange = Math.Abs(v1 - v2);
                        if (v1 < v2 || double.IsNaN(v1) || double.IsNegativeInfinity(v1))
                            return CompareType.LessThen;
                        else if (v1 > v2 || double.IsPositiveInfinity(v1))
                            return CompareType.GreaterThen;
                        else
                            return CompareType.EqualTo;
                    } 
                default:
                    throw new Exception("Unknown DataType");
            }
        }

        public string ReadMemoryAsString(int address, DataType dataType, int? dataTypeSz)
        {            
            byte[] buffer = ReadMemoryAsBytes(address, dataType, dataTypeSz);         

            switch (dataType)
            {
                case DataType.Int32:
                    return BitConverter.ToInt32(buffer, 0).ToString();
                case DataType.Int16:
                    return BitConverter.ToInt16(buffer, 0).ToString();
                case DataType.Byte:
                    return ((sbyte)buffer[0]).ToString();
                case DataType.UInt32:
                    return BitConverter.ToUInt32(buffer, 0).ToString();
                case DataType.UInt16:
                    return BitConverter.ToUInt16(buffer, 0).ToString();
                case DataType.UByte:
                    return ((byte)buffer[0]).ToString();
                case DataType.String:                   
                    ASCIIEncoding encoding = new ASCIIEncoding();
                    return encoding.GetString(buffer);                    
                case DataType.Float:
                    return BitConverter.ToSingle(buffer, 0).ToString();
                case DataType.Double:
                    return BitConverter.ToDouble(buffer, 0).ToString();
                default:
                    throw new Exception("Unknown DataType");
            }

        }

        public byte[] ReadMemoryAsBytes(int address, DataType dataType, int? dataTypeSz)
        {
            int sz;
            if (dataTypeSz == null)
                sz = DataTypeByteCount(dataType);
            else
                sz = dataTypeSz.Value;
            byte[] buffer = new byte[sz];
            try
            {
                int lengthRead;
                int baseAddress = Process.MainModule.BaseAddress.ToInt32();
                int length = Process.MainModule.ModuleMemorySize;

                IntPtr handle = Process.Handle;

                if (address >= baseAddress && address <= baseAddress + length)
                {
                    ReadProcessMemory(handle, address, buffer, buffer.Length, out lengthRead);
                }
            }
            catch (Exception ex)
            {
                buffer = new byte[sz];
            }
            return buffer;
        }

        public int WriteValue(int address, byte[] values)
        {
            int lengthWrite = 0;
            try
            {
                IntPtr handle = Process.Handle;
                WriteProcessMemory(handle, address, values, values.Length, out lengthWrite);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
            return lengthWrite;
        }        

        private int DataTypeByteCount(DataType dataType)
        {            
            switch (dataType)
            {
                case DataType.Int32:
                   return sizeof(Int32);                   
                case DataType.Int16:
                   return sizeof(Int16);
                case DataType.Byte:
                   return sizeof(sbyte);
                case DataType.UInt32:
                   return sizeof(UInt32);
                case DataType.UInt16:
                   return sizeof(UInt16);
                case DataType.UByte:
                   return sizeof(byte);
                case DataType.String:
                    throw new NotImplementedException();                    
                case DataType.Float:
                    return sizeof(float);
                case DataType.Double:
                    return sizeof(double);
                default:
                    throw new Exception("Unknown DataType");
            }          
        }

        private float UpdateProgress(int start, int end, float lastPrecentDone, int currentAddress, int addressFoundCount)
        {
            float dx = end - start;
            float dy = currentAddress - start;
            float precentDone = (float)(dy / dx);

            //this must change by at least one precent or equal 100%
            if (precentDone - lastPrecentDone > .1f || end == currentAddress)
            {
                _parent.Invoke(new ThreadStart(delegate()
                {
                    if (OnProgressChange != null)
                    {
                        OnProgressChange(this, new SearchUpdateEventArgs(start, end, currentAddress, addressFoundCount));
                    }
                }));
                lastPrecentDone = precentDone;
            }
            return lastPrecentDone;
        }
    }   
}
