﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace MemoryPatch
{

    public class SearchData
    {
        public bool FirstSearch { get; private set; }
        public SearchType SearchType { get; private set; }
        public byte[] Value1 { get; private set; }
        public float Difference { get; private set; }
        public DataType DataType { get; private set; }
        public int DataLength { get; private set; }
        public int SaveResultsIndex { get; private set; }


        private SearchData()
        {
        }

        public static SearchData CreateSearchData(bool firstSearch, SearchType searchType, DataType dataType, string value1, int saveResultIndex)
        {
            SearchData data = new SearchData();
            data.SaveResultsIndex = saveResultIndex;
            data.FirstSearch = firstSearch;
            data.SearchType = searchType;
            data.DataType = dataType;            

            bool oneVal = (searchType == SearchType.Excat);
            if (searchType == SearchType.HasIncreasedBy ||
                searchType == SearchType.HasDecreasedBy)
            {
                data.Difference = float.Parse(value1);
            }

            switch (dataType)
            {
                case DataType.Int32:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(Int32.Parse(value1));

                    data.DataLength = sizeof(int);
                    break;
                case DataType.Int16:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(Int16.Parse(value1));

                    data.DataLength = sizeof(short);
                    break;
                case DataType.Byte:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(SByte.Parse(value1));

                    data.DataLength = sizeof(byte);
                    break;
                case DataType.UInt32:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(UInt32.Parse(value1));

                    data.DataLength = sizeof(int);
                    break;
                case DataType.UInt16:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(UInt32.Parse(value1));

                    data.DataLength = sizeof(short);
                    break;
                case DataType.UByte:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(Byte.Parse(value1));

                    data.DataLength = sizeof(byte);
                    break;
                case DataType.String:
                    //ASCIIEncoding encoding = new ASCIIEncoding();
                    //if (oneVal)
                    //    Value1 = encoding.GetBytes(value1);

                    if (oneVal)
                    {
                        data.Value1 = new byte[value1.Length * sizeof(char)];
                        int x = 0;
                        for (int i = 0; i < value1.Length; i++)
                        {
                            byte[] bytes = BitConverter.GetBytes(value1[i]);
                            data.Value1[x++] = bytes[0];
                            data.Value1[x++] = bytes[1];
                        }
                        data.DataLength = data.Value1.Length;
                    }
                    else
                        throw new Exception("Can't search for an unkown string");

                    break;
                case DataType.Float:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(float.Parse(value1));
                    data.DataLength = sizeof(float);
                    break;
                case DataType.Double:
                    if (oneVal)
                        data.Value1 = BitConverter.GetBytes(double.Parse(value1));
                    data.DataLength = sizeof(double);
                    break;                
            }
            return data;
        }

    }

    public enum DataType
    {
        UByte,        
        UInt16,
        UInt32,
        Byte,          
        Int16,
        Int32,     
        String,
        Float,
        Double,
    }

    public enum SearchType
    {
        Excat,        
        UnKnown,
        HasIncreased,
        HasDecreased,
        HasNotChanged,
        HasChanged,
        HasDecreasedBy,
        HasIncreasedBy,
        SearchForAValue,
        SearchForBValue,
    }

    public enum CompareType
    {
        LessThen,
        EqualTo,
        GreaterThen,        
    }
}
