﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.Globalization;

namespace MemoryPatch
{
    [Serializable]
    public class SavedAddress
    {
        [XmlAttribute("name")]
        public string Name;
        public bool Locked; 
        public int Address;                  
        public byte[] Value;
        public DataType DataType;
        public int RowIndex;
        public string Category;        

        public string StringAddress
        {
            get
            {
                return string.Format("{0:X000000}", Address);
            }
            set
            {
                Address = Int32.Parse(value, System.Globalization.NumberStyles.AllowHexSpecifier, null);
            }
        }

        [XmlIgnore]
        public string StringValue
        {
            get
            {               
                switch (DataType)
                {
                    case DataType.Int32:
                        if (Value.Length != 4)
                            Array.Resize<Byte>(ref Value, 4);

                        return BitConverter.ToInt32(Value, 0).ToString();                    
                    case DataType.Int16:
                        if (Value.Length != 2)
                            Array.Resize<Byte>(ref Value, 2);

                        return BitConverter.ToInt16(Value, 0).ToString();                   
                    case DataType.Byte:
                        if (Value.Length != 1)
                            Array.Resize<Byte>(ref Value, 1);

                        return ((sbyte)Value[0]).ToString();                    
                    case DataType.UInt32:
                        if (Value.Length != 4)
                            Array.Resize<Byte>(ref Value, 4);

                        return BitConverter.ToUInt32(Value, 0).ToString();                    
                    case DataType.UInt16:
                        if (Value.Length != 2)
                            Array.Resize<Byte>(ref Value, 2);

                        return BitConverter.ToUInt16(Value, 0).ToString();                    
                    case DataType.UByte:
                        if (Value.Length != 1)
                            Array.Resize<Byte>(ref Value, 1);

                        return ((byte)Value[0]).ToString();                    
                    case DataType.String:
                        throw new NotImplementedException();
                    case DataType.Float:
                        if (Value.Length != 4)
                            Array.Resize<Byte>(ref Value, 4);

                        return BitConverter.ToSingle(Value, 0).ToString();
                    case DataType.Double:
                        if (Value.Length != 8)
                            Array.Resize<Byte>(ref Value, 8);

                        return BitConverter.ToDouble(Value, 0).ToString();
                    default:
                        throw new Exception("Unknown DataType");
                }
            }
            set
            {
                switch (DataType)
                {
                    case DataType.Int32:
                        Value = BitConverter.GetBytes(Int32.Parse(value));                        
                        break;
                    case DataType.Int16:
                        try
                        {
                            Value = BitConverter.GetBytes(Int16.Parse(value));
                        }
                        catch (FormatException ex)
                        {
                            Value = BitConverter.GetBytes(Int16.Parse(value, NumberStyles.AllowHexSpecifier));
                        }
                        catch (OverflowException)
                        {
                            Value = new byte[] { 255, 255 };
                        }                      
                        break;
                    case DataType.Byte:
                        try
                        {
                            Value = new byte[] { (byte)SByte.Parse(value.ToLower()) };
                        }
                        catch (FormatException ex)
                        {
                            Value = new byte[] { (byte)SByte.Parse(value.ToLower(), NumberStyles.AllowHexSpecifier)};
                        }
                        catch (OverflowException)
                        {
                            Value = new byte[] { 0 };
                        }
                        break;
                    case DataType.UInt32:
                        Value = BitConverter.GetBytes(UInt32.Parse(value));
                        break;
                    case DataType.UInt16:
                        try
                        {
                            Value = BitConverter.GetBytes(UInt16.Parse(value));
                        }
                        catch (FormatException ex)
                        {
                            Value = BitConverter.GetBytes(UInt16.Parse(value, System.Globalization.NumberStyles.AllowHexSpecifier));
                        }
                        catch (OverflowException)
                        {
                            Value = new byte[] { 255, 255 };
                        }
                        break;
                    case DataType.UByte:
                        try
                        {
                            Value = new byte[] {Byte.Parse(value.ToLower())};
                        }
                        catch (FormatException ex)
                        {
                            Value = new byte[] {Byte.Parse(value.ToLower(), System.Globalization.NumberStyles.AllowHexSpecifier)};
                        }
                        catch (OverflowException)
                        {
                            Value = new byte[] {255};
                        }
                        break;
                    case DataType.String:
                        ASCIIEncoding encoding = new ASCIIEncoding();
                        Value = encoding.GetBytes(value);                 
                        break;
                    case DataType.Float:
                        Value = BitConverter.GetBytes(float.Parse(value));                        
                        break;
                    case DataType.Double:
                        Value = BitConverter.GetBytes(double.Parse(value));
                        break;
                }          
            }
        }
    
        public SavedAddress()
        {

        }

        public SavedAddress(bool locked, string name, int address, string value, DataType dataType, string category)
        {                     
            DataType = dataType;
            StringValue = value;
            Address = address;
            Name = name;
            Locked = locked;
            Category = category;
        }

        public SavedAddress(bool locked, string name, string address, string value, DataType dataType, string category)
        {
            DataType = dataType;
            StringValue = value;
            StringAddress = address;
            Name = name;
            Locked = locked;
            Category = category;
        }

        public SavedAddress(bool locked, string name, int address, byte[] value, DataType dataType, string category)
        {
            DataType = dataType;
            Value = value;
            Address = address;
            Name = name;
            Locked = locked;
            Category = category;
        }

        //public SavedAddress(DataType dataType, byte[] value, int address, string name, bool locked, int rowIndex)
        //{
        //    DataType = dataType;
        //    Value = value;
        //    Address = address;
        //    Name = name;
        //    Locked = locked;
        //    RowIndex = rowIndex;
        //}
        public override string ToString()
        {
            return "(" + Locked + ") " + Name + " " + StringAddress + " = " + StringValue + " " + DataType;
        }
    }
}
